/* -*- Mode: C; tab-width: 2; indent-tabs-mode: t; c-basic-offset: 2 -*-
 *
 * LASER
 * Copyright (C) 2006 Benjamin Canou <bookeldor@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <gdk/gdk.h>

#ifndef __LASER_COMMON_H
#define __LASER_COMMON_H

/* constants *****************************************************************/

#define LASER_CONFIG_SHARE                      DATADIR G_DIR_SEPARATOR_S "laser" G_DIR_SEPARATOR_S
#define LASER_CONFIG_DEFAULT_RESTORE            TRUE
#define LASER_CONFIG_DEFAULT_ARROWS             TRUE
#define LASER_CONFIG_DEFAULT_SHOW_STATS         FALSE
#define LASER_CONFIG_DEFAULT_THEME              (LASER_CONFIG_SHARE "themes" G_DIR_SEPARATOR_S "Tango")
#define LASER_CONFIG_DEFAULT_LEVEL_SET          (LASER_CONFIG_SHARE "levels" G_DIR_SEPARATOR_S "tutorial")
#define LASER_CONFIG_DEFAULT_LEVEL              0
#define LASER_CONFIG_DEFAULT_COLOUR             "#AAAAAAFF"
#define LASER_CONFIG_DEFAULT_CHEAT              ""
#define LASER_CONFIG_DEFAULT_WIDTH              500
#define LASER_CONFIG_DEFAULT_HEIGHT             400
#define LASER_CONFIG_DEFAULT_EDITOR_WIDTH       700
#define LASER_CONFIG_DEFAULT_EDITOR_HEIGHT      600
#define LASER_CONFIG_DEFAULT_ACTIONS            0
#define LASER_CONFIG_DEFAULT_CLOCK              0

#define LASER_GCONF_PATH                  "/apps/laser"
#define LASER_GCONF_KEY_RESTORE           (LASER_GCONF_PATH "/" "restore")
#define LASER_GCONF_KEY_ARROWS            (LASER_GCONF_PATH "/" "arrows")
#define LASER_GCONF_KEY_SHOW_STATS        (LASER_GCONF_PATH "/" "stats")
#define LASER_GCONF_KEY_THEME             (LASER_GCONF_PATH "/" "theme")
#define LASER_GCONF_KEY_COLOUR            (LASER_GCONF_PATH "/" "colour")
#define LASER_GCONF_KEY_LEVEL_SET         (LASER_GCONF_PATH "/" "history-set")
#define LASER_GCONF_KEY_LEVEL             (LASER_GCONF_PATH "/" "history-level")
#define LASER_GCONF_KEY_ACTIONS           (LASER_GCONF_PATH "/" "history-actions")
#define LASER_GCONF_KEY_CLOCK             (LASER_GCONF_PATH "/" "history-clock")
#define LASER_GCONF_KEY_CHEAT             (LASER_GCONF_PATH "/" "history-clicks")
#define LASER_GCONF_KEY_WIDTH             (LASER_GCONF_PATH "/" "width")
#define LASER_GCONF_KEY_HEIGHT            (LASER_GCONF_PATH "/" "height")
#define LASER_GCONF_KEY_EDITOR_WIDTH      (LASER_GCONF_PATH "/" "editor-width")
#define LASER_GCONF_KEY_EDITOR_HEIGHT     (LASER_GCONF_PATH "/" "editor-height")

enum {
	TRANSPARENT,
	COLOUR_RED, COLOUR_GREEN, COLOUR_BLUE, COLOUR_VIOLET,
	COLOUR_YELLOW, COLOUR_WHITE, COLOUR_CYAN
};
#define COLOURS COLOUR_CYAN

enum {
	NONE,
	MAT_MIR, MAT_CROSS, MAT_DUP, MAT_MUX, MAT_TEE,
	MAP_WALL, MAP_BEAMER, MAP_TARGET, MAP_HIDER, MAP_DIR, MAP_DIODE
};
#define MATERIALS MAT_TEE

#define IS_MATERIAL(type)												\
	(type > NONE && type <= MATERIALS)
#define EDIT_ROTATION(type)																							\
	((type > NONE && type <= MATERIALS) || type == MAP_BEAMER || type == MAP_DIR || type == MAP_DIODE)
#define EDIT_POSITION(type)											\
	(type > NONE)

enum {
	DIR_LEFT, DIR_DOWNLEFT, DIR_DOWN, DIR_DOWNRIGHT,
	DIR_RIGHT, DIR_UPRIGHT, DIR_UP, DIR_UPLEFT
};

/* types *********************************************************************/

typedef struct item {
  guint16 type;
  guint8 colour;
  guint8 direction;
} item;

typedef struct level {
  GSList  *name_i18n;
  gchar*  name;
  GSList  *comment_i18n;
  gchar*  comment;
	gchar   *solution;
  gint    materials[MATERIALS];
  gint    width, height;
  gint    real_height;
	gint    difficulty;
  item    **map;
} level;

typedef struct  level_set {
  gchar*  file;
  GSList  *name_i18n;
  gchar*  name;
  GSList  *note_i18n;
  gchar*  note;
  level   *levels;
  int     nb_levels;
} level_set;

struct materials_asoc {
	char *name;
	int index;
};

/* prototypes ****************************************************************/

level_set* level_set_new        (void);
level_set* dup_level_set        (level_set *level_set);
void       level_set_free       (level_set*);
void       free_i18n            (GSList *string_i18n);
GSList*    dup_i18n             (GSList *string_i18n, gchar* localised_orig, gchar** localised_dup);

typedef struct colour_t {
	/* cairo */
	gdouble r, g, b, a;
	/* gdk */
	GdkColor gdk_colour;
	guint16  gdk_alpha;
	/* string */
	gchar str[10];
} colour_t;

colour_t*  colour_new_from_string   (gchar str[10]);
colour_t*  colour_new_from_gdk      (GdkColor gdk_colour, guint16 gdk_alpha);
colour_t*  colour_new_from_cairo    (gdouble r, gdouble g, gdouble b, gdouble a);
void       colour_from_string   (colour_t *colour, gchar str[10]);
void       colour_from_gdk      (colour_t *colour, GdkColor gdk_colour, guint16 gdk_alpha);
void       colour_from_cairo    (colour_t *colour, gdouble r, gdouble g, gdouble b, gdouble a);

gboolean   file_error_dialog    (gchar *file, gchar *details);

/* globals ********************************************************************/

struct materials_asoc materials_assoc[MATERIALS];

/* generated C marshallers ***************************************************/

#include "laser-marshal.h"

#endif /* __LASER_COMMON_H */
