/* -*- Mode: C; tab-width: 2; indent-tabs-mode: t; c-basic-offset: 2 -*-
 *
 * LASER
 * Copyright (C) 2006 Benjamin Canou <bookeldor@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __LASER_CONFIG_H
#define __LASER_CONFIG_H

#include "config.h"

#include <glib.h>
#include <cairo.h>

#include "laser-common.h"

/* types **********************************************************************/

typedef struct _LaserConfig LaserConfig;
typedef struct _LaserConfigClass LaserConfigClass;
typedef struct _LaserConfigPrivate LaserConfigPrivate;

struct _LaserConfig {
  GObject parent;

	/*< private >*/
	LaserConfigPrivate* priv;
};

struct _LaserConfigClass {
  GObjectClass parent;

	void (*laser_config_colour_changed) (LaserConfig *laser_config, colour_t *colour);
	void (*laser_config_restore_changed) (LaserConfig *laser_config, gboolean restore);
	void (*laser_config_arrows_changed) (LaserConfig *laser_config, gboolean arrows);
	void (*laser_config_show_stats_changed) (LaserConfig *laser_config, gboolean show_stats);
	void (*laser_config_theme_changed) (LaserConfig *laser_config);
};

#define LASER_CONFIG_TYPE            (laser_config_get_type ())
#define LASER_CONFIG(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), LASER_CONFIG_TYPE, LaserConfig))
#define LASER_CONFIG_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), LASER_CONFIG_TYPE, LaserConfigClass))
#define IS_LASER_CONFIG(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), LASER_CONFIG_TYPE))
#define IS_LASER_CONFIG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), LASER_CONFIG_TYPE))
#define LASER_CONFIG_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), LASER_CONFIG_TYPE, LaserConfigClass))

typedef struct theme_cache theme_cache;

/* prototypes *****************************************************************/

GType            laser_config_get_type          (void);
LaserConfig*     laser_config_new               (void);
void             laser_config_destroy           (LaserConfig *config);

theme_cache*     sprite_cache_new               (LaserConfig *self, guint size);
void             sprite_cache_set               (theme_cache *cache, guint size);
cairo_surface_t* sprite_cache_get               (theme_cache *cache, item *item);

GSList*          get_available_themes           (void);
void             free_available_themes          (GSList *themes);

GSList*          get_available_level_sets       (void);
void             free_available_level_sets      (GSList *sets);

void             laser_config_set_theme         (LaserConfig *self, gchar *theme);
gchar*           laser_config_get_theme         (LaserConfig *self);

void             laser_config_set_cheat         (LaserConfig *self, gchar* code);
gchar*           laser_config_get_cheat         (LaserConfig *self);
void             laser_config_save_history      (LaserConfig *self);

void             laser_config_set_height        (LaserConfig *self, gint height);
gint             laser_config_get_height        (LaserConfig *self);

void             laser_config_set_width         (LaserConfig *self, gint width);
gint             laser_config_get_width         (LaserConfig *self);

void             laser_config_set_editor_height (LaserConfig *self, gint height);
gint             laser_config_get_editor_height (LaserConfig *self);

void             laser_config_set_editor_width  (LaserConfig *self, gint width);
gint             laser_config_get_editor_width  (LaserConfig *self);

void             laser_config_get_colour        (LaserConfig *self, colour_t *colour);
void             laser_config_set_colour        (LaserConfig *self, colour_t *colour);

void             laser_config_colour_from_id    (guchar id, colour_t *colour);

void             laser_config_set_restore       (LaserConfig *self, gboolean restore);
gboolean         laser_config_get_restore       (LaserConfig *self);

void             laser_config_set_arrows        (LaserConfig *self, gboolean arrows);
gboolean         laser_config_get_arrows        (LaserConfig *self);

void             laser_config_set_show_stats    (LaserConfig *self, gboolean show_stats);
gboolean         laser_config_get_show_stats    (LaserConfig *self);

void             laser_config_set_level         (LaserConfig *self, gchar *set, gint level);
void             laser_config_get_level         (LaserConfig *self, gchar **set, gint *level);

void             laser_config_set_actions       (LaserConfig *self, gint actions);
gint             laser_config_get_actions       (LaserConfig *self);

void             laser_config_set_clock         (LaserConfig *self, gint clock);
gint             laser_config_get_clock         (LaserConfig *self);


#endif /* __LASER_CONFIG_H */
