/* -*- Mode: C; tab-width: 2; indent-tabs-mode: t; c-basic-offset: 2 -*-
 *
 * LASER
 * Copyright (C) 2006 Benjamin Canou <bookeldor@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <libgnomevfs/gnome-vfs.h>
#include "laser-common.h"

#ifndef __LASER_PARSER_H
#define __LASER_PARSER_H

/* constants *****************************************************************/

#define READ_BLOCK 100

/* types *********************************************************************/

typedef struct stream {
	/* state */
	gboolean       error;
	gchar*         message;
	guint          col, row;
	/* internal */
	GnomeVFSHandle *handle;
	gchar          buffer[1000];
	gint           i;
	guint          col_hist[1000];
	gint           col_i;
	gboolean       eof, was_eof;
} stream_t;

/* prototypes ****************************************************************/

stream_t*  stream_new              (gchar *uri);
gchar      stream_get              (stream_t *stream);
void       stream_put              (stream_t *stream,	gchar c);
void       stream_skip_blanks      (stream_t *stream);
gboolean   stream_read_keyword     (stream_t *stream,	gchar **k);
gboolean   stream_read_string      (stream_t *stream, gchar **s, gboolean alloc);
gboolean   stream_read_string_i18n (stream_t *stream,	GSList **s_i18n, gchar **s, gboolean alloc);
void       stream_destroy          (stream_t *stream);

void       parse_level_set_check   (gchar *uri, GError **gerror, gint *n_levels);
void       parse_level_set_meta    (gchar *uri,
																		GSList **name_i18n, gchar **name,
																		GSList **note_i18n, gchar **note,
																		GError **gerror);
void       parse_level_set_levels  (gchar *uri, GSList **names, GError **gerror);
void       parse_level_set         (gchar *uri, level_set *level_set, GError **gerror);

GSList*    parse_cheat_code        (gchar *code, gboolean *err);


#endif /* __LASER_PARSER_H */
