/* -*- Mode: C; tab-width: 2; indent-tabs-mode: t; c-basic-offset: 2 -*-
 *
 * LASER
 * Copyright (C) 2006 Benjamin Canou <bookeldor@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <glib.h>
#include <glib/gi18n.h>
#include <stdarg.h>
#include <libgnomevfs/gnome-vfs.h>
#include "laser-common.h"
#include "laser-writer.h"

gchar* escape (gchar *str) {
	int sz = strlen (str) + 1;
	for (gchar *s = str;*s != '\0';s++) {
		if (*s == '"' || *s == '\\') {
			sz++;
		}
	}
	gchar *r = g_malloc (sz);
	gchar *sr = r;
	for (gchar *s = str;*s != '\0';s++) {
		if (*s == '"' || *s == '\\') {
			*sr++ = '\\';
		}
		*sr++ = *s;
	}
	*sr = '\0';
	return r;
}

void print (GnomeVFSHandle *handle, gchar *str, ...) {
	va_list  pa;
	GnomeVFSFileSize sz;

	va_start (pa, str);
	for (gchar *s = str;*s;s++) {
		switch (*s) {
		case '%' : {
			s++;
			switch (*s) {
			case '\0' :
				break;
			case 's' : {
				gchar *fs = escape (va_arg (pa, gchar*));
				gnome_vfs_write (handle, fs, strlen (fs), &sz);
				g_free (fs);
				break;
			}
			case 'd' : {
				gint x = va_arg (pa, gint);
				gint sign = (x >= 0) ? 1 : -1;
				x *= sign;
				gchar buf[100];
				gint n = 99;
				buf[n] = '\0';
				if (x == 0) {
					n--;
					buf[n] = '0';
				}
				while (x != 0 && n > 1) {
					n--;
					buf[n] = (x % 10) + '0';
					x /= 10;
				}
				if (sign == -1) {
					n--;
					buf[n] = '-';
				}
				gnome_vfs_write (handle, &buf[n], 99 - n, &sz);
				break;
			}
			case 'i' : {
				GSList *is = va_arg (pa, GSList*);
				for (GSList *l = is;l != NULL;l = g_slist_next (l)) {
					gchar **pair = (gchar**)l->data;
					if (strcmp (pair[0], "C") == 0) {
						gchar guil = '"';
						gnome_vfs_write (handle, &guil, 1, &sz);
						gchar *es = escape (pair[1]);
						gnome_vfs_write (handle, es, strlen (es), &sz);
						g_free (es);
						gnome_vfs_write (handle, &guil, 1, &sz);
					}
				}
				for (GSList *l = is;l != NULL;l = g_slist_next (l)) {
					gchar **pair = (gchar**)l->data;
					if (strcmp (pair[0], "C") != 0) {
						gchar *a = " [", *b = "] \"", *c = "\"";
						gnome_vfs_write (handle, a, 2, &sz);
						gchar *es = escape (pair[0]);
						gnome_vfs_write (handle, es, strlen (es), &sz);
						g_free (es);
						gnome_vfs_write (handle, b, 3, &sz);
						es = escape (pair[1]);
						gnome_vfs_write (handle, es, strlen (es), &sz);
						g_free (es);
						gnome_vfs_write (handle, c, 1, &sz);
					}
				}
				break;
			}
			case 'c' : {
				gchar c = va_arg (pa, int);
				gnome_vfs_write (handle, &c, 1, &sz);
				break;
			}
			default :
				gnome_vfs_write (handle, s, 1, &sz);
				break;
			}
			break;
		}
		default :
			gnome_vfs_write (handle, s, 1, &sz);
			break;
		}
	}
	va_end (pa);
}

/**
 * write_level_set:
 *
 */
void write_level_set (level_set *level_set, gchar *uri, GError **err) {
	GnomeVFSHandle *handle;
	GnomeVFSResult r =
		gnome_vfs_create (&handle, uri,
											GNOME_VFS_OPEN_WRITE | GNOME_VFS_OPEN_TRUNCATE,
											FALSE, 0644);
	if (r != GNOME_VFS_OK) {
		*err = g_error_new_literal (0, 0, gnome_vfs_result_to_string (r));
		return;
	}

	int i, j, k;

  print (handle, "Set %i\n", level_set->name_i18n);
  print (handle, "Note %i\n", level_set->note_i18n);

  for (i = 0;i < level_set->nb_levels;i++) {
    print (handle, "Level %i\n", level_set->levels[i].name_i18n);
    print (handle, "  Comment %i\n", level_set->levels[i].comment_i18n);
    print (handle, "  Difficulty %d\n", level_set->levels[i].difficulty);
		if (level_set->levels[i].solution != NULL) {
			print (handle, "  Solution \"%s\"\n", level_set->levels[i].solution);
		}
    for (j = 0;j < MATERIALS;j++) {
			if (level_set->levels[i].materials[j] > 0) {
				print (handle, "  %s %d\n",
							 materials_assoc[j].name,
							 level_set->levels[i].materials[j]);
			}
    }
    print (handle, "  Map\n  /");
    for(j = 0;j < level_set->levels[i].width;j++) {
      print (handle, "--+");
    }
    print (handle, "\n  ");
    for (j = 0; j < level_set->levels[i].real_height;j++) {
      for(k = 0;k < level_set->levels[i].width;k++) {
				print (handle, "|");
				if(level_set->levels[i].map[j][k].type == NONE) {
					print (handle, "  ");
				} else {
					if(level_set->levels[i].map[j][k].type == MAP_WALL) {
						print (handle, "##");
					} else {
						if(level_set->levels[i].map[j][k].type == MAP_HIDER) {
							print (handle, "HH");
						} else {
							if (level_set->levels[i].map[j][k].type == MAP_DIODE) {
								print (handle, "D");
							} else {
								switch (level_set->levels[i].map[j][k].colour) {
								case COLOUR_RED :
									print (handle, "r");
									break;
								case COLOUR_GREEN :
									print (handle, "g");
									break;
								case COLOUR_BLUE :
									print (handle, "b");
									break;
								case COLOUR_VIOLET :
									print (handle, "v");
									break;
								case COLOUR_WHITE :
									print (handle, "w");
									break;
								case COLOUR_YELLOW :
									print (handle, "y");
									break;
								case COLOUR_CYAN :
									print (handle, "c");
									break;
								default :
									print (handle, " ");
								break;
								}
							}
							switch (level_set->levels[i].map[j][k].type) {
							case MAP_BEAMER :
							case MAP_DIODE :
								switch (level_set->levels[i].map[j][k].direction) {
								case DIR_RIGHT :
									print (handle, ">");
									break;
								case DIR_LEFT :
									print (handle, "<");
									break;
								case DIR_UP :
									print (handle, "^");
									break;
								case DIR_DOWN :
									print (handle, "v");
									break;
								default :
									print (handle, "%d", level_set->levels[i].map[j][k].direction);
									break;
								}
								break;
							case MAP_TARGET :
								print (handle, "@");
								break;
							default :
								print (handle, " ");
								break;
							}
						}
					}
				}
      }
      print (handle, "|\n  ");
      for(k = 0;k < level_set->levels[i].width;k++) {
				print (handle, "+--");
      }
      if (j == level_set->levels[i].real_height - 1) {
				print (handle, "/");
      } else {
				print (handle, "+");
      }
      print (handle, "\n  ");
    }
    print (handle, "EndLevel\n");
  }
  print (handle, "EndSet\n");

	gnome_vfs_close (handle);
}
